# -*- coding: utf-8 -*-
from setuphelpers import *
from setupdevhelpers import *


def update_package():
    # Declaring local variables
    package_updated = False
    proxies = get_proxies()
    if not proxies:
        proxies = get_proxies_from_wapt_console()
    download_url = "https://www.nirsoft.net/utils/iconsext_setup.exe"
    latest_bin = download_url.split("/")[-1]
    latest_bin = "iconsext_setup.exe"

    # Deleting binaries
    remove_outdated_binaries("*")

    # Downloading latest binaries
    print("Download URL is: %s" % download_url)
    if not isfile(latest_bin):
        print("Downloading: %s" % latest_bin)
        wget(download_url, latest_bin, proxies=proxies)
    else:
        print("Binary is present: %s" % latest_bin)

    # Checking version from file
    bin_temp = "iconsext.exe"
    unzip_with_7zip(latest_bin, ".", bin_temp)
    if get_os_name() == "Windows" and "windows" in control.target_os.lower():
        version = get_version_from_binary(bin_temp)
    else:
        version = control.get_software_version()
    remove_file(bin_temp)

    # Changing version of the package
    if Version(version, 4) > Version(control.get_software_version(), 4):
        print("Software version updated (from: %s to: %s)" % (control.get_software_version(), Version(version)))
        package_updated = True
    else:
        print("Software version up-to-date (%s)" % Version(version))
    control.set_software_version(version)
    control.save_control_to_wapt()

    # Validating update-package-sources
    return package_updated

    # # Changing version of the package and validating update-package-sources
    # return complete_control_version(control, version)


def unzip_with_7zip(filename, target=None, filenames=[], extract_with_full_paths=True, recursive=True):
    r"""Extract the files from an archive file with 7zip with patterns in filenames to target directory

    Args:
        filename (str): path to archive file. (can be relative to temporary unzip location of package)
        target (str): archive content to target dir location (dir will be created if needed). Default: dirname(archive file) + basename(archive file)
        filenames (str or list of str): list of filenames / glob patterns (path sep is normally a slash)
        extract_with_full_paths (bool): keeping or not the subfolders of the archive as is
        recursive (bool): looking or not for filenames recursively

    Returns:
        None

    .. versionadded:: 2.0

    .. versionchanged:: 2.2
        changed default target location to make it correspond with unzip()

    """
    if not target:
        target = makepath(os.path.dirname(os.path.abspath(filename)), os.path.splitext(os.path.basename(filename))[0])
    if not isinstance(filenames, list):
        filenames = [filenames]

    sevenzip_path = get_7z_path()

    if not filenames:
        if extract_with_full_paths:
            run(r'"%s" x "%s" %s %s -aoa' % (sevenzip_path, filename, "" if not target else '-o"%s"' % target, "" if not recursive else "-r"))
        else:
            run(r'"%s" e "%s" %s %s -aoa' % (sevenzip_path, filename, "" if not target else '-o"%s"' % target, "" if not recursive else "-r"))
    else:
        for extract in filenames:
            if extract_with_full_paths:
                run(
                    r'"%s" x "%s" %s "%s" %s -aoa'
                    % (sevenzip_path, filename, "" if not target else '-o"%s"' % target, extract, "" if not recursive else "-r")
                )
            else:
                run(
                    r'"%s" e "%s" %s "%s" %s -aoa'
                    % (sevenzip_path, filename, "" if not target else '-o"%s"' % target, extract, "" if not recursive else "-r")
                )


def get_7z_path():
    """Check if the 7zip binary path exists and returns it.

    Returns:
        str: path to the 7zip binary
    """

    if "Windows" in get_os_name():
        sevenzip_path = makepath(programfiles, "7-Zip", "7z.exe")
        if isfile(sevenzip_path):
            return sevenzip_path
        sevenzip_path = makepath(programfiles32, "7-Zip", "7z.exe")
        if isfile(sevenzip_path):
            return sevenzip_path
        error('Could not find path to 7zip binary, please install "tis-7zip"')
    else:
        sevenzip_path = makepath("/usr", "local", "bin", "7zz")
        if isfile(sevenzip_path):
            return sevenzip_path
        sevenzip_path = sevenzip_path.replace("7zz", "7z")
        if isfile(sevenzip_path):
            return sevenzip_path
        error('Could not find path to 7zip binary, please install "tis-7zip"')
